import gulp from 'gulp';
import rev from 'gulp-rev';
import cached from 'gulp-cached';
import rename from 'gulp-rename';
import postCSS from 'gulp-postcss';
import sourcemaps from 'gulp-sourcemaps';

import conf from './conf';

export function serveMainCSS () {
    const p = [
        require('postcss-import')(),
        require('postcss-extend')(), // @extend, placeholder
        require('postcss-nested')(), // nested rules like SASS
        require('postcss-custom-media')(), // custom media
        require('postcss-preset-env')(), // autoprefixer, CSS 4
        require('postcss-critical-split')({ // critical CSS
            'output': 'rest'
        }),
        require('css-mqpacker')(), // merge media queries
        require('cssnano')({ // minify CSS; Disable autoprefixer, because it's already included in cssnext
            autoprefixer: false,
            zindex: false
        }),
        require('postcss-reporter')(), // report issue in console
        require('postcss-browser-reporter')() // report issue in browser
    ];

    const logError = (error) => {
        console.log(error.toString());
        this.emit('end');
    };

    let stream = gulp.src(`wp-content/themes/${conf.name}/css/main.css`);

    if (!conf.isProduction) {
        stream = stream
            .pipe(sourcemaps.init())
            .on('error', logError);
    }

    stream = stream
        .pipe(postCSS(p))
        .on('error', logError)
        .pipe(rename({ suffix: '.min' }))
        .on('error', logError);

    if (conf.isProduction) {
        stream = stream
            .pipe(rev());
    } else {
        stream = stream
            .pipe(cached('css-main'))
            .on('error', logError)
            .pipe(sourcemaps.write('.'))
            .on('error', logError);
    }

    stream = stream
        .pipe(gulp.dest(`wp-content/themes/${conf.name}/css/`))
        .on('error', logError);

    return stream;
}
