<?php

/**
 * Fired during plugin activation
 *
 * @link       https://www.biuro.lt
 * @since      1.0.0
 *
 * @package    Biuro_Contacts
 * @subpackage Biuro_Contacts/includes
 */

/**
 * Fired during plugin activation.
 *
 * This class defines all code necessary to run during the plugin's activation.
 *
 * @since      1.0.0
 * @package    Biuro_Contacts
 * @subpackage Biuro_Contacts/includes
 * @author     Biuro <info@biuro.lt>
 */
class Biuro_Contacts_Activator {

	/**
	 * Short Description. (use period)
	 *
	 * Long Description.
	 *
	 * @since    1.0.0
	 */
	public static function activate() {
        global $wpdb;

        if ( is_multisite() ) {
            // Get all blogs in the network and activate plugin on each one
            $blog_ids = $wpdb->get_col( "SELECT blog_id FROM $wpdb->blogs" );
            foreach ( $blog_ids as $blog_id ) {
                switch_to_blog( $blog_id );
                self::create_db_employees();
                restore_current_blog();
            }
        } else {
            // create_table();
            self::create_db_employees();
        }


	}

    public static function create_db_employees() {
        global $wpdb;

        $table_name = $wpdb->prefix . "biuro_employees";
        $version = get_option( 'biuro-contacts_db_version', '1.0' );

        if( $wpdb->get_var( "show tables like '{$table_name}'" ) != $table_name ||
            version_compare( $version, '1.0' ) < 0 ) {
            $charset_collate = $wpdb->get_charset_collate();
            $sql[] = "CREATE TABLE " . $table_name . " (
                id mediumint(9) NOT NULL AUTO_INCREMENT,
                created datetime DEFAULT '0000-00-00 00:00:00',
                name tinytext,
                email varchar(128) DEFAULT '',
                phone varchar(128) DEFAULT '',
                city varchar(255) DEFAULT null,
                city_id int DEFAULT null,
                message longtext DEFAULT null,
                cv text DEFAULT null,
                updated datetime DEFAULT '0000-00-00 00:00:00',
                confirm tinyint DEFAULT 0,
                job_id int DEFAULT null,
                PRIMARY KEY (id)
            ) $charset_collate;";

            require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );
            /**
             * It seems IF NOT EXISTS isn't needed if you're using dbDelta - if the table already exists it'll
             * compare the schema and update it instead of overwriting the whole table.
             *
             * @link https://code.tutsplus.com/tutorials/custom-database-tables-maintaining-the-database--wp-28455
             */
            dbDelta( $sql );
            add_option( 'biuro-contacts_db_version', $plugin_name_db_version );
        }
    }

}
