<?php
/*
Plugin Name: Cookies warning
Description: Biuro cookies warning plugin
Author: Biuro
Version: 1.0
Author URI: https://www.biuro.lt/
*/


class Cookies_Warning extends WP_Widget {
    // Main constructor
    public function __construct() {
        parent::__construct(
            'cookies-policy',
            __( 'Cookies Warning | Biuro', 'biuro' ),
            array(
                'customize_selective_refresh' => true,
            )
        );
    }

    // The widget form (for the backend )
    public function form( $instance ) {
        // Set widget defaults
        $defaults = array(
            'agree'    => '',
            'close'    => '',
            'content' => '',
        );

        // Parse current settings with defaults
        extract( wp_parse_args( ( array ) $instance, $defaults ) ); ?>

        <?php // Content Field ?>
        <p>
            <label for="<?php echo esc_attr( $this->get_field_id( 'content' ) ); ?>"><?php _e( 'Content:', 'biuro' ); ?></label>
            <textarea class="widefat" id="<?php echo esc_attr( $this->get_field_id( 'content' ) ); ?>" name="<?php echo esc_attr( $this->get_field_name( 'content' ) ); ?>" rows="12"><?php echo wp_kses_post( $content ); ?></textarea>
        </p>

        <?php // Agree Field ?>
        <p>
            <label for="<?php echo esc_attr( $this->get_field_id( 'agree' ) ); ?>"><?php _e( 'Agree:', 'biuro' ); ?></label>
            <input class="widefat" id="<?php echo esc_attr( $this->get_field_id( 'agree' ) ); ?>" name="<?php echo esc_attr( $this->get_field_name( 'agree' ) ); ?>" type="text" value="<?php echo esc_attr( $agree ); ?>" />
        </p>

        <?php // Close Field ?>
        <p>
            <label for="<?php echo esc_attr( $this->get_field_id( 'close' ) ); ?>"><?php _e( 'Close:', 'biuro' ); ?></label>
            <input class="widefat" id="<?php echo esc_attr( $this->get_field_id( 'close' ) ); ?>" name="<?php echo esc_attr( $this->get_field_name( 'close' ) ); ?>" type="text" value="<?php echo esc_attr( $close ); ?>" />
        </p>

    <?php }

    // Update widget settings
    public function update( $new_instance, $old_instance ) {
        $instance = $old_instance;
        $instance['agree']    = isset( $new_instance['agree'] ) ? wp_strip_all_tags( $new_instance['agree'] ) : '';
        $instance['close']     = isset( $new_instance['close'] ) ? wp_strip_all_tags( $new_instance['close'] ) : '';
        $instance['content'] = isset( $new_instance['content'] ) ? wp_kses_post( $new_instance['content'] ) : '';
        return $instance;
    }
    // Display the widget
    public function widget( $args, $instance ) {
        extract( $args );
        // Check the widget options
        $agree     = isset( $instance['agree'] ) ? $instance['agree'] : '';
        $close     = isset( $instance['close'] ) ? $instance['close'] : '';
        $content = isset( $instance['content'] ) ?$instance['content'] : '';

        // WordPress core before_widget hook (always include )
        echo $before_widget;
        // Display the widget
        echo '<div id="cookie-warning" class="c-cookies-warning">';

            echo '<div class="l-inner">';

            // Display content field
            if ( $content ) {
                echo $content;
            }
            echo '<p class="c-cookies-warning--actions">';

            // Display agree button
            if ( $agree ) {
                echo '<button id="cookie-agree" class="o-btn c-btn--main" type="button" title="' . $agree . '">' . $agree . '</button>';
            }

            // Display close button
            if ( $close ) {
                echo '<button id="cookie-close" class="o-btn c-btn--slim" type="button" title="' . $close . '">' . $close . '</button>';
            }
            echo '</p>';


            echo '</div>';

        echo '</div>';
        // WordPress core after_widget hook (always include )
        echo $after_widget;
    }
}


// register Cookies_Warning
add_action( 'widgets_init', function(){
    register_widget( 'Cookies_Warning' );
});

