<?php
/*
Plugin Name: Data controller
Description: Biuro data controller plugin
Author: Biuro
Version: 1.0
Author URI: https://www.biuro.lt/
*/


class Data_Controller extends WP_Widget {
    // Main constructor
    public function __construct() {
        parent::__construct(
            'data-controller',
            __( 'Data Controller | Biuro', 'biuro' ),
            array(
                'customize_selective_refresh' => true,
            )
        );
    }

    // The widget form (for the backend )
    public function form( $instance ) {
        // Set widget defaults
        $defaults = array(
            'content' => '',
        );

        // Parse current settings with defaults
        extract( wp_parse_args( ( array ) $instance, $defaults ) ); ?>

        <?php // Content Field ?>
        <p>
            <label for="<?php echo esc_attr( $this->get_field_id( 'content' ) ); ?>"><?php _e( 'Content:', 'biuro' ); ?></label>
            <textarea class="widefat" id="<?php echo esc_attr( $this->get_field_id( 'content' ) ); ?>" name="<?php echo esc_attr( $this->get_field_name( 'content' ) ); ?>" rows="12"><?php echo wp_kses_post( $content ); ?></textarea>
        </p>

    <?php }

    // Update widget settings
    public function update( $new_instance, $old_instance ) {
        $instance = $old_instance;
        $instance['content'] = isset( $new_instance['content'] ) ? wp_kses_post( $new_instance['content'] ) : '';
        return $instance;
    }

    // Display the widget
    public function widget( $args, $instance ) {
        extract( $args );

        // Check the widget options
        $content = isset( $instance['content'] ) ?$instance['content'] : '';

        // WordPress core before_widget hook (always include )
        echo $before_widget;
        // Display the widget
        echo '<div class="c-data-controller">';

            // Display content field
            if ( $content ) {
                echo $content;
            }

        echo '</div>';

        // WordPress core after_widget hook (always include )
        echo $after_widget;
    }
}


// register Data_Controller
add_action( 'widgets_init', function(){
    register_widget( 'Data_Controller' );
});

