<?php

class JsonDataCollector
{

    // Job properties data map
    protected $properties = [
        'livas-id'      => 'id',
        'valid'         => 'galioja',
        'post_date'     => 'data',
        'post_modified' => 'atnaujinta',
        'title'         => 'pareigos',
        'post_content'  => 'aprasymas',
        'slug'          => 'seo_url',
        'contact'       => 'contacts',
        'salary_from'   => 'atlygis_nuo',
        'salary_to'     => 'atlygis_iki',
        'video_url'     => 'video_url',
    ];

    // Taxonomies data map
    protected $terms = [
        'city'    => [
            'slug' => 'miestas_id',
            'name' => 'regionas',
            '__primary' => true,
            'additionalCities' => [
                    'slug' => 'city_id',
                    'name' => 'city',
            ]
        ],
        'field'   => [
            'slug' => 'kategorija',
            'name' => 'sritis',
            '__primary' => true,
            'additionalCategories' => [
                'slug' => 'category_id',
                'name' => 'category_name',
            ]
        ],
        'company' => [
            'name' => 'imone'
        ],
        'type'    => [
            'types' =>  [
                'slug' => 'type_id',
                'name' => 'type_name'
            ]
        ],
    ];

    protected $seoProperties = [
        'metadesc' => 'seo_description'
    ];

    // Custom posts data map
    protected $posts = [
        'division' => [
            'title' => 'padalinys_name',
            'slug'  => 'padalinys'
        ],
    ];

    // Language map
    protected $lang = 'kalba';

    // Relations mapping
    protected $relationsMap = [
        'division-id' => 'division',
    ];

    // Source data
    protected $sourceData;

    protected $data;

    public function __construct($source)
    {
        $this->setData($source);
        $this->data = $this->transformData();
    }

    /**
     *  Set data source:
     *  get data from source and transform into array
     *
     * @param $source
     */
    protected function setData($source)
    {
        $this->sourceData = $this->loadData($source);
    }

    /**
     *  Parse xml data
     *
     * @param $source
     *
     * @return SimpleXMLElement
     */
    protected function loadData($source)
    {
        $context = stream_context_create([
            'http' => [
                'header' => 'Accept: application/json',
            ],
            'ssl'  => [
                'verify_peer'      => false,
                'verify_peer_name' => false,
            ],
        ]);
        $json = file_get_contents($source, false, $context);
        return json_decode($json);
    }

    /**
     *  Get data
     *
     * @return mixed
     */
    public function getData()
    {
        return $this->data;
    }

    protected function transformData()
    {
        $result = [];
        foreach ($this->sourceData as $ad) {
            $item = [];

            $item['properties'] = $this->getProperties($ad);
            $item['lang'] = $ad->{$this->lang};
            $item['terms'] = $this->getTerms($ad);
            $item['posts'] = $this->getPosts($ad);
            $item['relations'] = $this->relationsMap;
            $item['seo_properties'] = $this->getSeoProperties($ad);

            $result[] = $item;
        };

        return $result;
    }

    private function getProperties($ad)
    {
        $result = [];

        foreach ($this->properties as $propertyKey => $propertyValue) {
            if (isset($ad->{$propertyValue})) {
                $dataValue = $ad->{$propertyValue};
            }
            $result[$propertyKey] = $this->getValue($dataValue);
        }

        return $result;
    }

    private function getSeoProperties($ad)
    {
        $result = [];

        foreach ($this->seoProperties as $propertyKey => $propertyValue) {
            if (isset($ad->{$propertyValue})) {
                $dataValue = $ad->{$propertyValue};
            }
            $result[$propertyKey] = $this->getValue($dataValue);
        }

        return $result;
    }

    private function getTerms($ad)
    {
        $result = [];

        foreach ($this->terms as $termKey => $termProperties) {

            $entity = $this->getEntity($ad, $termProperties);


            if ( ! empty($entity)) {

                $result[$termKey][] = $entity;
            }

            $appends = array_filter($termProperties, function($item) {
                return is_array($item);
            });

            foreach ($appends as $propertyKey => $propertyValues) {

                if (isset($ad->{$propertyKey}) && !empty($ad->{$propertyKey})) {
                    foreach ($ad->{$propertyKey} as $data) {
                        $result[$termKey][] = $this->getEntity($data, $propertyValues);
                    }
                }
            }
        }

        return $result;
    }

    private function getEntity($ad, $propertyMap)
    {
        $entity = [];
        foreach ($propertyMap as $propertyKey => $propertyValue) {

            if ( ! is_array($propertyValue)) {

                if (preg_match('/^\_\_/', $propertyKey)) {
                    $entity[$propertyKey] = $propertyValue;
                }
                elseif (isset($ad->{$propertyValue})) {
                    $dataValue = $ad->{$propertyValue};
                    $entity[$propertyKey] = $this->getValue($dataValue);
                }
            }
        }

        return $entity;
    }

    private function getPosts($ad)
    {
        $result = [];

        foreach ($this->posts as $postKey => $postProperties) {
            foreach ($postProperties as $propertyKey => $propertyValue) {
                if (isset($ad->{$propertyValue})) {
                    $dataValue = $ad->{$propertyValue};
                    $result[$postKey][$propertyKey] = $this->getValue($dataValue);
                }
            }
        }

        return $result;
    }

    private function getValue($value)
    {
        return (is_array($value)) ? $value : html_entity_decode($value);
    }
}
